#include <stdio.h>
#include <stdlib.h>
#include "portaudio.h"
#include "tone.h"

#define SAMPLE_RATE  (8000)
#define NUM_CHANNELS    (1)
#define FRAMES_PER_BUFFER  (64) // needs to be shorter (in time) than delays
#define TONELEN 250  // length in ms
#define TONEGAP 200 // gap in ms

#ifndef DEMO
#define DEMO  1   // set to 0 to build library
#endif


/* Select sample format. */
#define PA_SAMPLE_TYPE  paFloat32
typedef float SAMPLE;

#define PI 3.1415926536

/* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().
*/

#define NFREQ 3
static float frequency[NFREQ] = { 0.0, 0.0, 0.0};
static float amp[NFREQ] = { 0.4, 0.4, 0.2 };

int setAmplitude(float ampin,int n)
{
  if (n<0||n>=NFREQ) return 1;
  amp[n]=ampin;
  return 0;
}

int setFrequency(float freq, int n)
{
  if (freq>SAMPLE_RATE/2 || freq<0 || n<0 || n>=NFREQ) return 1;
  frequency[n]=freq;
  return 0;
}

static int playCallback( void *inputBuffer, void *outputBuffer,
                         unsigned long framesPerBuffer,
                         PaTimestamp outTime, void *userData )
{
  static unsigned t=0;
  unsigned long n=0;
  int fn;
  SAMPLE *p=(SAMPLE *)outputBuffer;
  for (n=0;n<framesPerBuffer;n++)
    {
      p[n]=0;
      for (fn=0;fn<NFREQ;fn++)
	{
	if (frequency[fn]!=SAMPLE_SILENCE && amp[fn]!=0.0) 
	  p[n]+=amp[fn]*(sin(t*2*PI*frequency[fn]/SAMPLE_RATE));  
	}
      if (++t==SAMPLE_RATE) t=0;
    }

  return 0;
}


void audioEnd(AUDIOHANDLE stream)
{
    Pa_CloseStream( (PortAudioStream *)stream );
    Pa_Terminate();
}

int audioInit(AUDIOHANDLE *stream)
{
    PortAudioStream *str;
    PaError    err;
    int        i;
    err = Pa_Initialize();
    if( err != paNoError ) goto error;
    err = Pa_OpenStream(
         &str,
              paNoDevice,
              0,               /* NO input */
              PA_SAMPLE_TYPE,
              NULL,
              Pa_GetDefaultOutputDeviceID(),
              NUM_CHANNELS,
              PA_SAMPLE_TYPE,
              NULL,
              SAMPLE_RATE,
              FRAMES_PER_BUFFER,            /* frames per buffer */
              0,               /* number of buffers, if zero then use default minimum */
              paClipOff,       /* we won't output out of range samples so don't bother clipping them */
              playCallback,
              NULL );
    if( err != paNoError ) goto error;
    *stream=(AUDIOHANDLE)str;
    if( *stream )
    {
        err = Pa_StartStream( *stream );
        if( err != paNoError ) goto error;
    }
    return (int)err;

error:
    Pa_Terminate();
    return (int)err;

}

static float tone1[]={ 1336.0, 1209.0, 1336.0, 1477.0, 1209.0, 1336.0, 1477.0, 1209.0, 1336.0, 1447.0, 1209.0, 1477.0, 1633.0, 1633.0, 1633.0, 1633.0  };
static float tone2[]={ 941.0, 697.0, 697.0, 697.0, 770.0, 770.0, 770.0, 852.0, 852.0, 852.0, 941.0, 941.0, 679.0, 770.0, 852.0, 941.0};

void audioDial(int i)
{
  int n=-1;
  if (i>=0 && i<=15) n=i;
  if (i=='*') n=10;
  if (i=='#') n=11;
  if (i>='0' && i<='9') n=i-'0'; 
  if (i>='A' && i<='D') n=i-'A'+12;
  if (i>='a' && i<='d') n=i-'a'+12;
  if (n==-1) return;
  setFrequency(tone1[n],0);
  setFrequency(tone2[n],1);
  Pa_Sleep(TONELEN); // length of tones
  setFrequency(SAMPLE_SILENCE,0);
  setFrequency(SAMPLE_SILENCE,1);
  Pa_Sleep(TONEGAP);  // pause between digits
}

#if DEMO

int main(int argc, char *argv[])
{
  char *num="5551212";
  int i;
  AUDIOHANDLE stream;
  audioInit(&stream);
  if (argc>1) num=argv[1];
  setAmplitude(0.3,0);
  setAmplitude(0.3,1);
  for (i=0;i<strlen(num);i++)
    {
      putchar(num[i]);
      fflush(stdout);
      audioDial(num[i]);
    }
  audioEnd(stream);
  putchar('\n');
  return 0;
}

#endif

